SystemOrganization addCategory: #'FileSystem-AnsiStreams'!

Object subclass: #FileSystemStream
	instanceVariableNames: 'handle position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-AnsiStreams'!

!FileSystemStream commentStamp: '<historical>' prior: 0!
I am an abstract superclass for read- and write-streams that perform IO via a handle rather than by calling primitives directly. My subclasses' provide a cursor on a collection, so that sequences of IO messages can be position independent.

handle
	A subclass of FileSystemHandle. All IO goes through this object.
	
position
	An integer describing the next index to be accessed.!

FileSystemStream subclass: #FileSystemReadStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-AnsiStreams'!

!FileSystemReadStream commentStamp: 'cwp 11/18/2009 11:36' prior: 0!
I implement (more or less) the ANSI <FileReadStream> protocol. I provide methods for reading data from a file. !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/29/2009 22:45'!
atEnd
	^ position - 1 = handle size! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/29/2009 22:58'!
do: aBlock
	[self atEnd] whileFalse: [aBlock value: self next]! !

!FileSystemReadStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:28'!
isRead
	^ true! !

!FileSystemReadStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:28'!
isWrite
	^ false! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/29/2009 22:59'!
next
	| result |
	result := handle at: position.
	position := position + 1.
 	^ result! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'MarianoMartinezPeck 1/9/2012 18:02'!
next: count
	^ self nextInto: (ByteArray new: count)
! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'MarianoMartinezPeck 1/9/2012 18:02'!
next: number into: aCollection
    | count |
    count := handle at: position read: aCollection startingAt: 1 count: number.

    position := position + count.
     ^ count < aCollection size
        ifTrue: [aCollection first: count]
        ifFalse: [aCollection]! !

!FileSystemReadStream methodsFor: 'squeak' stamp: 'cwp 7/31/2009 00:24'!
nextInto: aCollection
	| count |
	count := handle at: position read: aCollection startingAt: 1 count: aCollection size.
	position := position + count.
 	^ count < aCollection size
		ifTrue: [aCollection first: count]
		ifFalse: [aCollection]! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/29/2009 23:31'!
nextLine
	| char |
	^ ByteArray streamContents:
		[:out |
		[self atEnd or: [#(13 10) includes: (char := self next)]] whileFalse: [out nextPut: char].
		(char = 13 and: [self peek = 10]) ifTrue: [position := position + 1]].
	! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/30/2009 23:03'!
nextMatchFor: anObject
	^ self next = anObject! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/29/2009 23:27'!
peek
	^ handle at: position! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/30/2009 23:06'!
peekFor: anObject
	^ self peek = anObject! !

!FileSystemReadStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/12/2011 20:49'!
size
	^ handle size! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/30/2009 23:19'!
skip: anInteger
	position := position + anInteger min: handle size + 1! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/30/2009 23:27'!
skipTo: anObject
	
	| result |
	[self atEnd or: [result := (self next = anObject)]] whileFalse.
	^ result! !

!FileSystemReadStream methodsFor: 'ansi gettable' stamp: 'cwp 7/30/2009 23:37'!
upTo: anObject
	
	| byte |
	^ ByteArray streamContents:
		[:out |
		[self atEnd or: [(byte := self next) = anObject]] whileFalse:
			[out nextPut: byte]]
		! !

!FileSystemStream class methodsFor: 'instance creation' stamp: 'cwp 7/29/2009 22:30'!
onHandle: aHandle
	^ self new setHandle: aHandle
	! !

!FileSystemStream methodsFor: 'public' stamp: 'cwp 7/29/2009 22:29'!
close
	handle close! !

!FileSystemStream methodsFor: 'public' stamp: 'cwp 7/29/2009 22:29'!
contents
	| size contents |
	size := handle size.
	contents := ByteArray new: size.
	handle at: 1 read: contents startingAt: 1 count: size.
	^ contents! !

!FileSystemStream methodsFor: 'public' stamp: 'cwp 10/29/2009 09:47'!
isOpen
	^ handle isOpen! !

!FileSystemStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:28'!
isRead
	self subclassResponsibility! !

!FileSystemStream methodsFor: 'public' stamp: 'cwp 2/5/2011 22:54'!
isStream
	^ true! !

!FileSystemStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:28'!
isWrite
	self subclassResponsibility! !

!FileSystemStream methodsFor: 'public' stamp: 'cwp 10/29/2009 09:47'!
open
	handle open! !

!FileSystemStream methodsFor: 'public' stamp: 'Camillobruni 8/18/2011 16:43'!
position
	^ position! !

!FileSystemStream methodsFor: 'public' stamp: 'Camillobruni 8/18/2011 16:49'!
position: anInteger
	self assert: anInteger > 0.
	position := anInteger.! !

!FileSystemStream methodsFor: 'initialize-release' stamp: 'cwp 7/29/2009 22:30'!
setHandle: aHandle
	handle := aHandle.
	position := 1! !

FileSystemStream subclass: #FileSystemWriteStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-AnsiStreams'!

!FileSystemWriteStream commentStamp: 'cwp 11/18/2009 11:36' prior: 0!
I implement (more or less) the ANSI <FileWriteStream> protocol. I provide methods for writing data to a file.!

!FileSystemWriteStream methodsFor: 'writing' stamp: 'CamilloBruni 8/12/2011 14:26'!
<< aCollection
	^ self nextPutAll: aCollection! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'cwp 7/28/2009 22:25'!
cr
	self nextPut: 13! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:35'!
crlf
	"Append a line feed character to the receiver."

	self nextPut: 13; nextPut: 10! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 1/24/2012 13:54'!
crtab
	"Append a return character, followed by a single tab character, to the 
	receiver."

	self nextPut: 13.
	self nextPut: Character tab! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:36'!
crtab: anInteger 
	"Append a return character, followed by anInteger tab characters, to the 
	receiver."

	self nextPut: 13.
	anInteger timesRepeat: [self tab]! !

!FileSystemWriteStream methodsFor: 'ansi puttable' stamp: 'cwp 7/28/2009 22:22'!
flush
	handle flush! !

!FileSystemWriteStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:28'!
isRead
	^ false! !

!FileSystemWriteStream methodsFor: 'accessing' stamp: 'CamilloBruni 8/14/2011 15:29'!
isWrite
	^ true! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:35'!
lf
	"Append a line feed character to the receiver."

	self nextPut: 10! !

!FileSystemWriteStream methodsFor: 'writing' stamp: 'cwp 7/23/2009 17:03'!
nextPut: anObject
	handle at: position put: anObject.
	position := position + 1.
	^ anObject! !

!FileSystemWriteStream methodsFor: 'writing' stamp: 'cwp 7/23/2009 22:51'!
nextPutAll: aCollection
	handle at: position write: aCollection startingAt: 1 count: aCollection size.
	position := position + aCollection size.
	^ aCollection! !

!FileSystemWriteStream methodsFor: 'writing' stamp: 'CamilloBruni 1/24/2012 13:54'!
print: anObject
	"Have anObject print itself on the receiver."

	anObject printOn: self! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:36'!
space
	"Append a space character to the receiver."

	self nextPut: 32! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:24'!
space: anInteger 
	"Append anInteger space characters to the receiver."

	anInteger timesRepeat: [self space]! !

!FileSystemWriteStream methodsFor: 'writing' stamp: 'CamilloBruni 1/24/2012 13:42'!
store: anObject 
	"Have anObject print on the receiver for purposes of rereading."

	anObject storeOn: self! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'cwp 7/28/2009 22:28'!
tab
	self nextPut: 9! !

!FileSystemWriteStream methodsFor: 'character writing' stamp: 'CamilloBruni 8/12/2011 14:24'!
tab: anInteger 
	"Append anInteger tab characters to the receiver."

	anInteger timesRepeat: [self tab]! !

!FileSystemWriteStream methodsFor: 'file' stamp: 'cwp 10/15/2009 22:05'!
truncate
	handle truncateTo: position - 1! !

!FileSystemWriteStream methodsFor: 'file' stamp: 'cwp 10/15/2009 21:49'!
truncateTo: anInteger
	handle truncateTo: anInteger! !
